const express = require('express');
const app = express();
const fs = require('fs');
const WavEncoder = require('wav-encoder');
const bodyParser = require('body-parser');
const request = require('request');
const dotenv = require('dotenv');
dotenv.config();

app.use(express.static(__dirname + '/public'));
app.use(bodyParser.json({ extended: true}));
app.use(bodyParser.raw({ type: 'application/octet-stream', limit: '100mb' }))

// サーバー開始
const port = process.env.PORT || 8888;
const server = require('http').createServer({}, app);
server.listen(port, () => {
  console.log('Server listening on port %s', port);
});

app.post("/stopRecord", async (req, res) => {
  await exportAsWAV(req.body);
  res.send('OK');
});

app.get('/speechToText', async (req, res) => {
  let text = await speechToText();
  res.json({ message: text });
});

app.post('/summarize', async (req, res) => {
  const text = req.body.text;
  const lineNumber = req.body.lineNumber;
  let summary = await summarize(text, lineNumber);
  res.json({ message: summary });
});

app.post('/textToSpeech', async (req, res) => {
  const text = req.body.text;
  await textToSpeech(text);
  res.send('OK');
})

app.get('/download', (req, res) => {
  res.download('text-to-speech.mp3');
});

function exportAsWAV(data) {
  let _data = new Float32Array(data.buffer, data.byteOffset, data.length / Float32Array.BYTES_PER_ELEMENT);

  let audioData = {
    sampleRate: 44100,
    channelData: [_data]
  };

  return new Promise((resolve, reject) => {
    WavEncoder.encode(audioData).then((buffer) => {
      fs.writeFile('record.wav', Buffer.from(buffer), (error) => {
        if (error) {
          console.log(error)
        } else {
          resolve();
        }
      });
    });
  })
}

function summarize(text, lineNumber) {
  return new Promise((resolve, reject) => {
    const options = {
      'method': 'POST',
      'url': 'https://api.a3rt.recruit-tech.co.jp/text_summarization/v1?apikey=XXXXXXXXXXXXXXXXXXXXXXX',
      'headers': {
        'Content-Type': 'application/x-www-form-urlencoded'
      },
      form: {
        sentences: text,
        linenumber: lineNumber,
      }
    }

    request(options, (error, response, body) => {
      let result;
      const _body = JSON.parse(response.body);
      if (_body.status === 0) {
        result = _body.summary.join("。") + "。";
      } else {
        result = _body.message;
      }
      resolve(result);
    });
  });
}

function speechToText() {
  return new Promise(async (resolve, reject) => {
    const speech = require('@google-cloud/speech');
    const client = new speech.SpeechClient();
    const request = {
      config: {
        enableAutomaticPunctuation: true,
        encoding: "LINEAR16",
        languageCode: "ja-JP",
        model: "default",
        sampleRateHertz: 44100,
      }
    };

    let texts = [];
    const recognizeStream = client.streamingRecognize(request)
      .on('error', console.error)
      .on('data', data => {
        let text = data.results[0].alternatives[0].transcript;
        texts.push(text);
      })
      .on('end', () => {
        resolve(texts.join(''));
      });

    fs.createReadStream('record.wav').pipe(recognizeStream);
  });
}

async function textToSpeech(text) {
  const textToSpeech = require('@google-cloud/text-to-speech');
  const client = new textToSpeech.TextToSpeechClient();
  const util = require('util');
  const request = {
    input: { text: text },
    voice: { languageCode: 'ja-JP', ssmlGender: 'NEUTRAL' },
    audioConfig: { audioEncoding: 'MP3' },
  };

  try {
    const [response] = await client.synthesizeSpeech(request);
    const writeFile = util.promisify(fs.writeFile);
    await writeFile('text-to-speech.mp3', response.audioContent, 'binary');
  } catch (e) {
    console.log(e.message);
  }
}
